from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from .core import Carrier


def main():
    Carrier().run()


if __name__ == "__main__":
    main()
